/*
 * Copyright by Marco Savegnago.  All rights reserved.
 * This software may be freely distributed and used, but it may not
 * under any circumstances be sold by anyone other than the author.
 * It may be distributed by a commercial company as long as it is
 * for no cost.
 *
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago             	Data di creazione : 21/1/2000
 * Codice   :
 *
 * AX25 Address class implementation
 */

#include "ax25.h"
#include <mem.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

/*******************************************************************
	Copy contructor
*******************************************************************/

TAX25Address::TAX25Address(const TAX25Address& other)
{
	memcpy(&callsign, other.callsign, CMAXCLEN);
	ssid=other.ssid | CAX25AddressFlag_reserved_Mask;
}

TAX25Address& TAX25Address::operator =(const TAX25Address& other)
{
	memcpy(&callsign, other.callsign, CMAXCLEN);
	ssid=other.ssid | CAX25AddressFlag_reserved_Mask;

	return *this;
}

TAX25Address::TAX25Address(const char* address)
{
	if(*address)
		setAX25Address(address);
}

TAX25Address::TAX25Address(const byte* address)
{
	if(*address)
		setAX25Address(address);
}

TAX25Address& TAX25Address::operator=(const char *address)
{
	if(*address)
		setAX25Address(address);

	return *this;
}

void TAX25Address::setAX25Address(const byte *address)
{
	memcpy(&callsign, address, CMAXCLEN);
	ssid=*(address+CMAXCLEN) & CAX25AddressFlag_SSID_Mask;
}

byte* TAX25Address::getAX25Address(byte *address, int maxLen)
{
	if(maxLen<(CMAXALEN))
	{
		*address=EOS;
		return NULL;
	}

	memcpy(address, callsign,CMAXCLEN);

	address[CMAXCLEN] = ssid;

	return	address;
}


char *TAX25Address::getAX25Address(char *address, int withSSID) const
{
	/* translate callsign */

	char *d=address;

	for(int i=0; i<CMAXCLEN; i++)
	{
		char c = callsign[i];

		if(c == ' ')
				break;
		else
			*d++ = c;
	}

	*d = '\0';

	if(withSSID)
	{
		if(NULL != (i = ( (ssid >> CAX25AddressShift) & CAX25AddressSSIDMax) ) )
			sprintf(d,"-%d",i);
	}

	return address;		/* return pointer to string */
}

/*******************************************************************
	Compare with another AX25 Address
*******************************************************************/

int TAX25Address::compare (const TAX25Address& compareWith, int withSSID) const
{
	/* compare the calls */

	if(memcmp(callsign, compareWith.callsign, CMAXCLEN))
											return false;

    /* and if the ssid must not be compared return */

    if(!withSSID)
    	return true;

	/* else compare the SSIDs too */

	return int ((ssid & CAX25AddressFlag_SSID_Mask) == (compareWith.ssid & CAX25AddressFlag_SSID_Mask));
}

int	TAX25Address::compare(void* first, void* second)
{
	return *((TAX25Address *)first) == *((TAX25Address *)second);
}


int TAX25Address::setAX25Address(const char *address)
{
	if(*address == NULL)
			return false;

    int len = strlen(address);

	for(int i=0; i<CMAXCLEN; i++)
	{
		if(*address && *address != '-' && !isspace(*address) && i < len)
				callsign[i] = toupper(*address++);
		else
				callsign[i] = ' ';	/* pad w/ spaces */
	}

	if(i > CMAXCLEN && *address && *address != '-')	/* call too long */
										return false;

	if(*address == '-')
	{
		i = atoi(address+1);

		if(i > CAX25AddressSSIDMax)
						return false;		/* invalid ssid */
	}
	else
		i = 0;				/* default */

	ssid = CAX25AddressFlag_reserved_Mask	| (i << CAX25AddressShift);

	return true;
}
