/*
 * Copyright by Marco Savegnago.  All rights reserved.
 * This software may be freely distributed and used, but it may not
 * under any circumstances be sold by anyone other than the author.
 * It may be distributed by a commercial company as long as it is
 * for no cost.
 *
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Marco Savegnago             	Data di creazione : 21/1/2000
 * Codice   :
 *
 * Programma di configurazione per il firmware UIDIGI
 *
 */

#include	"profile.h"
#include	"tdef.h"
#include	"ax25.h"
#include	<stdio.h>
#include	<string.h>
#include	<ctype.h>
#include	<io.h>

extern unsigned _stklen = 8192U;

const   byte    by_ThisVersionSignature[]                       =  {'U','I','D','I','G','I','1','7'};
const   char    csz_ThisVersion[]                               =  "1.7";

const   long    l_ConfigurationOffset                           =  59L;
const   char    csz_DefaultBinaryFilename[]                     =  "UIDIGI.BIN";

const   char    sz_DigipeaterCallsign[]                         =  "DigipeaterCallsign";
const   char    sz_DigipeaterAlias[]                            =  "DigipeaterAlias";
const   char    sz_BeaconDestination[]                          =  "BeaconDestination";
const   char    sz_Beacon1Path[]                                =  "Beacon1Path";
const   char    sz_Beacon2Path[]                                =  "Beacon2Path";
const   char    sz_Beacon3Path[]                                =  "Beacon3Path";
const   char    sz_NorthPath[]                                  =  "NorthPath";
const   char    sz_SouthPath[]                                  =  "SouthPath";
const   char    sz_EastPath[]                                   =  "EastPath";
const   char    sz_WestPath[]                                   =  "WestPath";
const   char    sz_BudList[]                                    =  "BudList";
const   char    sz_UIDIGICall[]                                 =  "UIDigiCall";
const   char    sz_UIFloodCall[]                                =  "UIFloodCall";
const   char    sz_UITraceCall[]                                =  "UITraceCall";
const   char    sz_TxDelay[]                                    =  "TxDelay";
const   char    sz_FullDuplex[]                                 =  "FullDuplex";
const   char    sz_PPersistance[]                               =  "PPersistance";
const   char    sz_SlotTime[]                                   =  "SlotTime";
const   char    sz_Frack[]                                      =  "Frack";
const   char    sz_Maxframe[]                                   =  "Maxframe";
const   char    sz_Retry[]                                      =  "Retry";
const   char    sz_RespTime[]                                   =  "RespTime";
const   char    sz_LinkCheck[]                                  =  "LinkCheck";
const   char    sz_Beacon1Interval[]                            =  "Beacon1Interval";
const   char    sz_Beacon2Interval[]                            =  "Beacon2Interval";
const   char    sz_Beacon3Interval[]                            =  "Beacon3Interval";
const   char    sz_DuplicateSuppression[]                       =  "DuplicateSuppression";
const   char    sz_DigipeaterEnable[]                           =  "DigipeaterEnable";
const   char    sz_LoopSuppression[]                            =  "LoopSuppression";
const   char    sz_HandleUISSID[]                               =  "HandleUISSID";
const   char    sz_ReplyToQuery[]                               =  "ReplyToQuery";
const   char    sz_UIFLOODOptions[]                             =  "UIFLOODOptions";
const   char    sz_UITRACEOptions[]                             =  "UITRACEOptions";
const	char	sz_UIDIGICallsubsitution[]                      =  "UIDIGICallSubstitution";
const   char    sz_SysopPassword[]                              =  "SysopPassword";
const   char    sz_InfoText[]                                   =  "InfoText";
const   char    sz_Beacon1Text[]                                =  "Beacon1Text";
const   char    sz_Beacon2Text[]                                =  "Beacon2Text";
const   char    sz_Beacon3Text[]                                =  "Beacon3Text";

struct  DigipeaterPath
{
    byte    by_digipeaterPath[CMAXDIGIS][CMAXALEN];
    byte    by_end;
};
struct	UIDIGICONFIG
{
    byte    by_signature[8];
    byte    by_callsign[CMAXALEN];
    byte    by_alias[CMAXCLEN];
    byte    by_aprsdestination[CMAXALEN];

    DigipeaterPath BeaconDestination[3];

    DigipeaterPath NorthPath;
    DigipeaterPath SouthPath;
    DigipeaterPath EastPath;
    DigipeaterPath WestPath;

    DigipeaterPath Uidigi;
    DigipeaterPath BudList;

    byte    by_uiFloodCall[CMAXALEN];
    byte    by_uiTraceCall[CMAXALEN];

    byte    by_TxDelay;
    byte    by_FullDuplex;
    word    w_PPersistance;
    word    w_SlotTime;
    word    w_Frack;
    word    w_MaxFrame;
    word    w_Retry;
    word    w_RespTime;
    word    w_linkCheck;
    word    w_level2DigipeaterEnable;
    word    w_level2APRSLoopSuppression;
    word    w_level2APRSDuplicateSuppression;
    word    w_uissid;
    word    w_beaconInterval[3];
    word    w_replyToAPRSQuery;
    word    w_UIFLOODOptions;
    word    w_UITRACEOptions;
    word    w_UIDIGICallSubstitution;
    byte    by_password[81];
    byte    by_information[81];
    byte    by_beaconText[3][71];
};

struct	UIDIGICONFIG	Configuration;

const char sz_ErrorReadingPath[]      =   "\aError: Reading path %s\n";
const char sz_ErrorReading[]          =   "\aError: Reading %s. Use default.\n";
const char sz_ErrorReadingFile[]      =   "\aError: Reading file %s.";
const char sz_ErrorWritingFile[]      =   "\aError: Writing file %s.";
const char sz_ErrorOpeningFile[]      =   "\aError: Unable to open file: %s!";
const char sz_ErrorFileSignature[]    =   "\aError: File signature doesn't match!\nVersion %s expected.";
const char sz_Banner[]                =   "UIDGCFG %s - (c) 1999~2000 Marco Savegnago %s\n";
const char sz_Usage[]                 =
"\nUIDIGI %s Configuration utility\n\n"

"Usage UIDGCFG [/|-command] <srcbinfile> <cfgtxtfile>\n\n"

"options:\n"
"\t/?|/H show this text\n"
;


/*
**  Check the source binary file signature
*/

int	CheckBinaryFileSignature(const char *psz_Filename)
{
	FILE*	fp;
	byte	by_thisFileSignature[8];

	if(NULL!=(fp=fopen(psz_Filename, "rb")))
	{
		fseek(fp,l_ConfigurationOffset,SEEK_SET);

		fread(&by_thisFileSignature, sizeof(by_thisFileSignature), 1, fp);

		fclose(fp);

		return	memcmp(&by_ThisVersionSignature, &by_thisFileSignature, 8);
	}

	return	-1;
}

/*
**  Read the source binary file configuration
*/

int	ReadBinaryFileConfig(const char *psz_Filename)
{
	FILE*	fp;

	if(NULL!=(fp=fopen(psz_Filename, "rb")))
	{
		fseek(fp,l_ConfigurationOffset,SEEK_SET);

		fread(&Configuration, sizeof(UIDIGICONFIG), 1, fp);

        fclose(fp);

		return 0;
	}

	return	-1;
}


/*
**  Read the source binary file configuration
*/

int	WriteBinaryFileConfig(const char *psz_Filename)
{
	FILE*	fp;

	if(NULL!=(fp=fopen(psz_Filename, "rb+")))
	{
		fseek(fp,l_ConfigurationOffset,SEEK_SET);

		fwrite(&Configuration, sizeof(UIDIGICONFIG), 1, fp);

        fclose(fp);

		return 0;
	}

	return	-1;
}

/*
**  Read and convert the Digipeater Path
*/

int	ExtractDigiPath(char* psz_Buffer, DigipeaterPath& path)
{
	TAX25Address	ax25Address;
	byte			by_address[CMAXALEN];
	char *p;
	int	i;

	for(i=0;i<CMAXDIGIS;i++)
	{
		memset(&path.by_digipeaterPath[i][0], 0, CMAXCLEN);
	}

	i = 0;

	p = strtok(psz_Buffer, ",");

	if(p)
	{
		while(p && i < CMAXDIGIS)
		{
			skipSpace(&p);

			ax25Address.setAX25Address(p);

			memcpy(&path.by_digipeaterPath[i][0],ax25Address.getAX25Address(by_address,CMAXALEN), CMAXALEN);

			i++;

			p=strtok(NULL, ",");
		}

        path.by_end=0;

		return 1;
	}

	return 0;
}

/*
**  Convert binary digipeater path to ascii digipeater
*/

int	BinPathToAsciiPath(DigipeaterPath& path, char *sz_list)
{
	TAX25Address	ax25Address;
	char			sz_buffer[128];

	if(path.by_digipeaterPath[0][0])
	{
		ax25Address.setAX25Address(path.by_digipeaterPath[0]);

		strcpy(sz_list,ax25Address.getAX25Address(sz_buffer, true));

		for(int i=1;i<CMAXDIGIS;i++)
		{
			if(path.by_digipeaterPath[i][0])
			{
				ax25Address.setAX25Address(&path.by_digipeaterPath[i][0]);
				strcat(sz_list, ",");
				strcat(sz_list,ax25Address.getAX25Address(sz_buffer, true));
			}
			else
				break;
		}

        return  1;
	}

    return 0;
}

/*
**  Read the Text Config
*/

int	ReadTextConfig(const char* pcc_ConfigName)
{
	TAX25Address	ax25Address;
	char			sz_buffer[128];
	char			sz_temp[256];
	char			sz_list[128];
	int				i;

	//	Read Digipeater Callsign AX25 address with SSID

	ax25Address.setAX25Address(Configuration.by_callsign);

	if(GetPrivateProfileString("", sz_DigipeaterCallsign, ax25Address.getAX25Address(sz_buffer, true), sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		ax25Address.setAX25Address(sz_temp);
		ax25Address.getAX25Address(Configuration.by_callsign);
	}

	//	Read Digipeater Alias string

	if(GetPrivateProfileString("", sz_DigipeaterAlias, Configuration.by_alias, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		memcpy(Configuration.by_alias,sz_temp, CMAXCLEN);
	}

	//	Read Digipeater Alias AX25 address with SSID

	ax25Address.setAX25Address(Configuration.by_aprsdestination);

	if(GetPrivateProfileString("", sz_BeaconDestination, ax25Address.getAX25Address(sz_buffer, true), sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		ax25Address.setAX25Address(sz_temp);
		ax25Address.getAX25Address(Configuration.by_aprsdestination);
	}

	//	Read Digipeater Beacon 1 Path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.BeaconDestination[0], sz_list);

	if(GetPrivateProfileString("", sz_Beacon1Path, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.BeaconDestination[0]))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater Beacon 2 Path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.BeaconDestination[1], sz_list);

	if(GetPrivateProfileString("", sz_Beacon2Path, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.BeaconDestination[1]))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater Beacon 3 Path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.BeaconDestination[2], sz_list);

	if(GetPrivateProfileString("", sz_Beacon3Path, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.BeaconDestination[2]))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater North Path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.NorthPath, sz_list);

	if(GetPrivateProfileString("", sz_NorthPath, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.NorthPath))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater South Path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.SouthPath, sz_list);

	if(GetPrivateProfileString("", sz_SouthPath, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.SouthPath))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater East Path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.EastPath, sz_list);

	if(GetPrivateProfileString("", sz_EastPath, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.EastPath))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater West Path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.WestPath, sz_list);

	if(GetPrivateProfileString("", sz_WestPath, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.WestPath))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater BudList

        BinPathToAsciiPath(Configuration.BudList, sz_list);

	if(GetPrivateProfileString("", sz_BudList, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.BudList))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Calls that digipeater repeat if inserted in a via list path AX25 addresses with SSID

        BinPathToAsciiPath(Configuration.Uidigi, sz_list);

	if(GetPrivateProfileString("", sz_UIDIGICall, sz_list, sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		if(!ExtractDigiPath(sz_temp, Configuration.Uidigi))
		{
			printf(sz_ErrorReadingPath, sz_temp);
			return	1;
		}
	}

	//	Read Digipeater UIFlood Callsign AX25 address with SSID

	ax25Address.setAX25Address(Configuration.by_uiFloodCall);

	if(GetPrivateProfileString("", sz_UIFloodCall, ax25Address.getAX25Address(sz_buffer, true), sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		ax25Address.setAX25Address(sz_temp);
		ax25Address.getAX25Address(Configuration.by_uiFloodCall);
	}

	//	Read Digipeater UITrace Callsign AX25 address with SSID

	ax25Address.setAX25Address(Configuration.by_uiTraceCall);

	if(GetPrivateProfileString("", sz_UITraceCall, ax25Address.getAX25Address(sz_buffer, true), sz_temp, sizeof(sz_temp), pcc_ConfigName))
	{
		ax25Address.setAX25Address(sz_temp);
		ax25Address.getAX25Address(Configuration.by_uiTraceCall);
	}

	//	Read Txdelay

	Configuration.by_TxDelay=GetPrivateProfileInt("", sz_TxDelay, Configuration.by_TxDelay, pcc_ConfigName);

	//	Read Fullduplex

	Configuration.by_FullDuplex=GetPrivateProfileInt("", sz_FullDuplex, Configuration.by_FullDuplex, pcc_ConfigName);

	//	Read PPersistence

	Configuration.w_PPersistance=GetPrivateProfileInt("", sz_PPersistance, Configuration.w_PPersistance, pcc_ConfigName);

	//	Read SlotTime

	Configuration.w_SlotTime=GetPrivateProfileInt("", sz_SlotTime, Configuration.w_SlotTime, pcc_ConfigName);

	//	Read Frack

	Configuration.w_Frack=GetPrivateProfileInt("", sz_Frack, Configuration.w_Frack, pcc_ConfigName);

	//	Read Maxframe

	Configuration.w_MaxFrame=GetPrivateProfileInt("", sz_Maxframe, Configuration.w_MaxFrame, pcc_ConfigName);

	//	Read Retry

	Configuration.w_Retry=GetPrivateProfileInt("", sz_Retry, Configuration.w_Retry, pcc_ConfigName);

	//	Read Level2 Response Timer

	Configuration.w_RespTime=GetPrivateProfileInt("", sz_RespTime, Configuration.w_RespTime, pcc_ConfigName);

	//	Read Link Check Timer

	Configuration.w_linkCheck=GetPrivateProfileInt("", sz_LinkCheck, Configuration.w_linkCheck, pcc_ConfigName);

	//	Read Digipeater Enable

	Configuration.w_level2DigipeaterEnable=GetPrivateProfileInt("", sz_DigipeaterEnable, Configuration.w_level2DigipeaterEnable, pcc_ConfigName);

    //  Read Level2 Digipeater Loop Suppression

    Configuration.w_level2APRSLoopSuppression=GetPrivateProfileInt("", sz_LoopSuppression, Configuration.w_level2APRSLoopSuppression, pcc_ConfigName);

    //  Read Level2 Digipeater UISSID Handling enable

    Configuration.w_uissid=GetPrivateProfileInt("", sz_HandleUISSID, Configuration.w_uissid, pcc_ConfigName);

	//	Read Level2 Digipeater Beacon 1 interval

	Configuration.w_beaconInterval[0]=GetPrivateProfileInt("", sz_Beacon1Interval, Configuration.w_beaconInterval[0], pcc_ConfigName);

	//	Read Level2 Digipeater Beacon 2 interval

	Configuration.w_beaconInterval[1]=GetPrivateProfileInt("", sz_Beacon2Interval, Configuration.w_beaconInterval[1], pcc_ConfigName);

	//	Read Level2 Digipeater Beacon 3 interval

	Configuration.w_beaconInterval[2]=GetPrivateProfileInt("", sz_Beacon3Interval, Configuration.w_beaconInterval[2], pcc_ConfigName);

	//	Read Level2 Digipeater Duplicate Suppression

	Configuration.w_level2APRSDuplicateSuppression=GetPrivateProfileInt("", sz_DuplicateSuppression, Configuration.w_level2APRSDuplicateSuppression, pcc_ConfigName);

	//	Read Level2 Digipeater APRS Query reply

	Configuration.w_replyToAPRSQuery=GetPrivateProfileInt("", sz_ReplyToQuery, Configuration.w_replyToAPRSQuery, pcc_ConfigName);

	//	Read UIFLOOD Options

	Configuration.w_UIFLOODOptions=GetPrivateProfileInt("", sz_UIFLOODOptions, Configuration.w_UIFLOODOptions, pcc_ConfigName);

	//	Read UITRACE Options

	Configuration.w_UITRACEOptions=GetPrivateProfileInt("", sz_UIFLOODOptions, Configuration.w_UITRACEOptions, pcc_ConfigName);

    //  Read UIDIGI Call substitution

    Configuration.w_UIDIGICallSubstitution=GetPrivateProfileInt("", sz_UIDIGICallsubsitution, Configuration.w_UIDIGICallSubstitution, pcc_ConfigName);

	//	Read Sysop Password

	if(!GetPrivateProfileString("", sz_SysopPassword, Configuration.by_password, Configuration.by_password, sizeof(Configuration.by_password), pcc_ConfigName))
    {
        printf(sz_ErrorReading, sz_SysopPassword);
    }

	//	Read Info Text

	if(!GetPrivateProfileString("", sz_InfoText, Configuration.by_information, Configuration.by_information, sizeof(Configuration.by_information), pcc_ConfigName))
    {
        printf(sz_ErrorReading, sz_InfoText);
    }

	//	Read Beacon 1 Text

	if(!GetPrivateProfileString("", sz_Beacon1Text, Configuration.by_beaconText[0], Configuration.by_beaconText[0], sizeof(Configuration.by_beaconText[0]), pcc_ConfigName))
    {
        printf(sz_ErrorReading, sz_Beacon1Text);
    }

	//	Read Beacon 2 Text

	if(!GetPrivateProfileString("", sz_Beacon2Text, Configuration.by_beaconText[1], Configuration.by_beaconText[1], sizeof(Configuration.by_beaconText[1]), pcc_ConfigName))
    {
        printf(sz_ErrorReading, sz_Beacon2Text);
    }

	//	Read Beacon 3 Text

	if(!GetPrivateProfileString("", sz_Beacon3Text, Configuration.by_beaconText[2], Configuration.by_beaconText[2], sizeof(Configuration.by_beaconText[2]), pcc_ConfigName))
    {
        printf(sz_ErrorReading, sz_Beacon3Text);
    }

	return	0;
}

/*
**  Print Usage
*/

void Usage (void)
{
	 printf(sz_Usage, csz_ThisVersion);
}

/*
**  The Main Thing
*/

int main(int na, char **a)
{
	int 	narg;
	char*	p;
	FILE*	fp;
	char*	psz_textConfigFileName = NULL;
	char*	psz_binaryConfigFileName = NULL;

	printf(sz_Banner, csz_ThisVersion, __DATE__);

	for ( narg = 1; narg < na; narg++ )
	{
		p=a[narg];

		if ( *p == '-' || *p == '/')
		{
			p++;

			switch(toupper(*p))
			{
				case '?':
				case 'H':
					Usage();
					return(-1);
				default:
					break;
			}
		}
		else if(psz_binaryConfigFileName == NULL)
			psz_binaryConfigFileName	=	p;
		else if(psz_textConfigFileName==NULL)
			psz_textConfigFileName	=	p;
		else
		{
			Usage();
			return	-1;
		}
	}

	if(na < 3)
	{
		Usage();
		return	-1;
	}

	//	Check if the files exists

	if(access(psz_binaryConfigFileName, 0))
	{
		printf(sz_ErrorOpeningFile, psz_binaryConfigFileName);
		return -1;
	}

	if(access(psz_textConfigFileName, 0))
	{
		printf(sz_ErrorOpeningFile, psz_textConfigFileName);
		return -1;
	}

	//	Check the binary file signature

	if(CheckBinaryFileSignature(psz_binaryConfigFileName))
	{
		printf(sz_ErrorFileSignature, csz_ThisVersion);
		return (1);
	}

    //	Read binary file configuration

	if(ReadBinaryFileConfig(psz_binaryConfigFileName))
	{
		printf(sz_ErrorReadingFile, psz_binaryConfigFileName);
		return	(2);
	}

	//	Read text file configuration

	if(ReadTextConfig(psz_textConfigFileName))
	{
		printf(sz_ErrorReadingFile, psz_textConfigFileName);
		return	(3);
	}

	//	Write binary file configuration

	if(WriteBinaryFileConfig(psz_binaryConfigFileName))
	{
		printf(sz_ErrorWritingFile, psz_textConfigFileName);
		return	(4);
	}

	return 0;
}
